---@class CraftSim
local CraftSim = select(2, ...)

---@class CraftSim.CraftRecipeData : CraftSim.CraftSimObject
CraftSim.CraftRecipeData = CraftSim.CraftSimObject:extend()

---@param recipeID number
function CraftSim.CraftRecipeData:new(recipeID)
    self.recipeID = recipeID
    self.numCrafts = 0
    self.totalProfit = 0
    self.totalExpectedProfit = 0
    self.numMulticraft = 0
    self.numMulticraftExtraItems = 0
    self.numResourcefulness = 0
    self.totalSavedCosts = 0
    self.totalExpectedSavedCosts = 0
    ---@type CraftSim.CraftResult[]
    self.craftResults = {}
    ---@type CraftSim.CraftResultItem[]
    self.totalItems = {}
    ---@type CraftSim.CraftResultSavedReagent[]
    self.totalSavedReagents = {}
end

---@param craftResult CraftSim.CraftResult
function CraftSim.CraftRecipeData:AddCraftResult(craftResult)
    self.numCrafts = self.numCrafts + 1
    self.totalProfit = self.totalProfit + craftResult.profit
    self.totalExpectedProfit = self.totalExpectedProfit + craftResult.expectedAverageProfit
    self.numMulticraft = self.numMulticraft + ((craftResult.triggeredMulticraft and 1) or 0)
    self.numResourcefulness = self.numResourcefulness + ((craftResult.triggeredResourcefulness and 1) or 0)
    self.totalSavedCosts = self.totalSavedCosts + craftResult.savedCosts
    self.totalExpectedSavedCosts = self.totalExpectedSavedCosts + craftResult.expectedAverageSavedCosts

    table.foreach(craftResult.craftResultItems, function(_, craftResultItemA)
        self.numMulticraftExtraItems = self.numMulticraftExtraItems + craftResultItemA.quantityMulticraft

        local craftResultItemB = CraftSim.GUTIL:Find(self.totalItems, function(craftResultItemB)
            local itemLinkA = craftResultItemA.item:GetItemLink() -- for gear its possible to match by itemlink
            local itemLinkB = craftResultItemB.item:GetItemLink()
            local itemIDA = craftResultItemA.item:GetItemID()
            local itemIDB = craftResultItemB.item:GetItemID()

            if itemLinkA and itemLinkB then -- if one or both are nil aka not loaded, we dont want to match..
                return itemLinkA == itemLinkB
            else
                return itemIDA == itemIDB
            end
        end)
        if craftResultItemB then
            craftResultItemB.quantity = craftResultItemB.quantity +
                (craftResultItemA.quantity + craftResultItemA.quantityMulticraft)
            craftResultItemB.quantityMulticraft = craftResultItemB.quantityMulticraft +
                craftResultItemA.quantityMulticraft
        else
            table.insert(self.totalItems, craftResultItemA:Copy())
        end
        -- we do not need to increase quantity as this would have been already done in the CraftSessionData Constructor
    end)

    table.foreach(craftResult.savedReagents, function(_, savedReagentA)
        local savedReagentB = CraftSim.GUTIL:Find(self.totalItems, function(savedReagentB)
            return savedReagentA.item:GetItemID() == savedReagentB.item:GetItemID()
        end)

        if not savedReagentB then
            table.insert(self.totalSavedReagents, savedReagentA)
        end
        -- we do not need to increase quantity as this would have been already done in the CraftSessionData Constructor
    end)

    table.insert(self.craftResults, craftResult)
end

function CraftSim.CraftRecipeData:Debug()
    local debugLines = {
        "recipeID: " .. self.recipeID,
        "numCrafts: " .. self.numCrafts,
        "totalProfit: " .. CraftSim.UTIL:FormatMoney(self.totalProfit, true),
        "totalExpectedProfit: " .. CraftSim.UTIL:FormatMoney(self.totalExpectedProfit, true),
        "numMulticraft: " .. self.numMulticraft,
        "numMulticraftExtraItems: " .. self.numMulticraftExtraItems,
        "numResourcefulness: " .. self.numResourcefulness,
        "totalSavedCosts: " .. CraftSim.UTIL:FormatMoney(self.totalSavedCosts),
        "totalExpectedSavedCosts: " .. CraftSim.UTIL:FormatMoney(self.totalExpectedSavedCosts),
    }
    return debugLines
end

function CraftSim.CraftRecipeData:GetJSON(indent)
    indent = indent or 0
    local jb = CraftSim.JSONBuilder(indent)
    jb:Begin()
    jb:Add("recipeID", self.recipeID)
    jb:Add("numCrafts", self.numCrafts)
    jb:Add("totalProfit", self.totalProfit)
    jb:Add("totalExpectedProfit", self.totalExpectedProfit)
    jb:Add("numMulticraft", self.numMulticraft)
    jb:Add("numMulticraftExtraItems", self.numMulticraftExtraItems)
    jb:Add("numResourcefulness", self.numResourcefulness)
    jb:Add("totalSavedCosts", self.totalSavedCosts)
    jb:AddList("totalItems", self.totalItems)
    jb:AddList("totalSavedReagents", self.totalSavedReagents)
    jb:AddList("craftResults", self.craftResults, true)
    jb:End()
    return jb.json
end
